# -*- coding: utf-8 -*-

"""
Azerothcore World of Warcraft Soap request and response Python library.
"""

from ._version import __version__

__short_description__ = "Azerothcore World of Warcraft Soap request and response Python library."
__license__ = "MIT"
__author__ = "Sanhe Hu"
__author_email__ = "husanhe@gmail.com"
__maintainer__ = "Sanhe Hu"
__maintainer_email__ = "husanhe@gmail.com"
__github_username__ = "MacHu-GWU"
